


# using TimeHelper as benchmark timer ...
from ot_my_libs.TimeHelper import TimeHelper as thd
import time


def benchmarkTimer():

    th = thd()

    bKey = th.benStart()

    print("making test run ...")
    time.sleep(1)

    th.benDone(
        bKey,
        f"Result of benStart / benDone [run at:{th.getNiceShortDate()}]", printIt=True )


# using db_helper
from ot_my_libs.db_helper import db_helper as dbh
class mdb ( dbh ):
    ## set your structure of db / table
    def set_struct(self):
        self.struct = {
            'tableA' : {
                      'lat':     "float",
                      'lon':     "float",
                      }
            }
def dbHelperExample():
    db = mdb('/tmp/mdbTest1.db')

    print("select tableA ... ")
    print( db.select('tableA') )

    print("inserting ....")
    insertRecordId = db.insert('tableA', {"lat":1.1,"lon":1.1} )

    print("select tableA ...")
    resA = db.select('tableA')
    print(resA)

    db.insert( 'tableA', {"lat":2.1,"lon":1.1} )
    print("select id,lat,lon from tableA where id=2 .....")
    print(db.select('tableA','id,lat,lon', 'id=2'))

    print("delete all ...")
    db.query('delete from tableA where 1')
    print("after delete ... select tableA")
    print(db.select('tableA'))


def testRunAll():
    print("can show some examples .....")
    print("- benchmarkTimer")
    benchmarkTimer()
    print("- db_helper making instance of db ...")
    dbHelperExample()

if __name__ == "__main__":
    testRunAll()



#!/usr/bin/env python3

from ot_my_libs.myLoger import myLoger as mlog
import ot_my_libs.myLoger as mlogD

import sys


lg = mlog( 'ArgsParse', silenc={
    'info': False,
    'debug': False,
    'error': True,
    'critical': True
    },defColor=mlogD.myLogColors.Yellow )

def ArgsParse( argv ):
    global lg
    
    lg.info(f"invoke ... argv:[{argv}]")
    
    tr = {}
    vS = 0
    lastArg = ""

    aName=""
    for i,a in enumerate( argv ):
        if i > 0:
            lg.debug("arg nr {0} arg({1})".format(i,a))
            if vS == 1:
                if aName != "":
                    tr[aName] = a
                    aName = ""
                    vS = 0
                else:
                    lg.error(f"Error empty value in arg:[{lastArg}]")
            elif a[0] == "-":
                vS = 1
                aName = a[1:]
                lastArg = aName

    return tr
    
if __name__ == "__main__":
    lg.info("This is a main of ArgsParse ... ")

    lg.info("* test 1 - as invoke from code use")
    a = ArgsParse(["","-h","1"])
    lg.info("res")
    lg.debug(a)
    lg.info(["res for h",a.get('h','')])

    lg.info("* test 2 - as regular use add some argument on start 'cat look for help'")
    a = ArgsParse( sys.argv )       
    lg.info("res")
    lg.debug(a)
    lg.info(["have help?", True if a.get('help', -1) != -1 else False])





import math
import copy
#from GPoint import *

class SPoint:
    def __init__(self,x=None,y=None,z=None):
        #print("type:[{}]".format(type(x)))
        if type(x) == type(list()):
            self.x = x[0]
            self.y = x[1]
            self.z = x[2]
        else: 
            self.x = x
            self.y = y
            self.z = z

class MyCalculate:
    
    def get3dDistance(self,p0,p1):
        pass
    
    def d3distance(self,v0, v1):  
        x1,y1,z1 = v0
        x2,y2,z2 = v1
        d = math.sqrt(math.pow(x2 - x1, 2) +
                 math.pow(y2 - y1, 2) +
                 math.pow(z2 - z1, 2)* 1.0) 
        print("Distance is ") 
        print(d)
        return d
    
    def getVectorAngles(self, p):
        #print("getVectorAngles p:",p)
        #print("xyz: {},{},{}".format(p.x,p.y,p.z))
        p0 = SPoint([.0,.0,.0])
        py10 = SPoint([.0,10.0,.0])
        
        
        x = self.angle(
            SPoint(p.y,p.z), 
            p0) 
        
        y = self.angle(
            SPoint(p.x,p.z), 
            p0)  
        
        z = self.angle(
            SPoint(p.x,p.y), 
            p0) 
        
        return [x,y,z] 
    
    
    def findIntersection(self, x1, y1, x2, y2, x3, y3, x4, y4 ):
        px= ( (x1*y2-y1*x2)*(x3-x4)-(x1-x2)*(x3*y4-y3*x4) ) / ( (x1-x2)*(y3-y4)-(y1-y2)*(x3-x4) ) 
        py= ( (x1*y2-y1*x2)*(y3-y4)-(y1-y2)*(x3*y4-y3*x4) ) / ( (x1-x2)*(y3-y4)-(y1-y2)*(x3-x4) )
        return [px, py]
    
    def distance(self, p0, p1):
        if p0.x and p0.y and p1.x and p1.y:
            return math.sqrt((p1.x - p0.x)**2 + (p1.y - p0.y)**2)
        return None
    
    ''' calculate angle from point 0,1,2=None
     points need to be SPoint
    '''
    def angle( self, p0, p1, p2 = None):
        if p2==None:
            #print("do angle from:",p0.x," x ",p0.y,"    to    ",p1.x," x ",p1.y)
            deltax = p1.x- p0.x
            deltay = p1.y- p0.y
            a = math.atan2(deltay,deltax)
            return math.degrees(a)
            
        else:
            a0 = self.angle(p1, p2)
            a1 = self.angle( p1, p0)
            if a0>=a1:
                a =  a0 - a1
            else:
                a = a1 - a0 
            return a
    
    def newPoint(self, 
        pStart, pEnd, 
        dist, 
        addAngle=0.0,
        returnAngle = False
        ):
        
        
        ang = self.angle(pStart, pEnd)+addAngle
        xn = pStart.x + dist*math.cos( math.radians(ang) )
        yn = pStart.y + dist*math.sin( math.radians(ang))
        
        pc = copy.copy(pEnd)
        pc.x = xn
        pc.y = yn
        
        if returnAngle:
            print("ang",ang)
            return pc,ang
        else:
            return pc
        
    
    
    def scale(self, a, b):
        return a/b
    
    def scaleForOffset(self, xo, yo, scale, step, tpx, tpy ):
        w = tpx
        h = tpy
        
        xoff = ((xo-w)/scale)
        yoff = ((yo-h)/scale)
        
        scale*=step
        
        xo = (xoff * scale)+w
        yo = (yoff * scale)+h
    
        return [scale, xo, yo]
    
    def mRound(self, val, accu):
        return round(val,accu)
    
    
    def remap(self, outMin,outMax, inMin,inMax, val):
        #print("remap",outMin,outMax, inMin,inMax, val)
        o = outMax - outMin
        i = inMax - inMin
        v = val - inMin
        vS = v/i
        
        #print('o',o)
        #print('i',i)
        #print('vS',vS)
        return outMin + (o*vS)
        
    
    
        




from ot_my_libs.TimeHelper import TimeHelper
from ot_my_libs.myLoger import *
import paho.mqtt.client as mqtt
import time
import sys



class myMqttClient:

    def __init__(self,
            clientId,
            host,
            port,
            on_connect=None,
            on_message=None,
            subscribe = ["#"]
            ):
        self.l = myLoger("myMqttClient", {
            'info': True,
            'debug': False,
            'error': False,
            'critical': True
            })
        self.ready = False
        self.th = TimeHelper()
        self.clientId = clientId
        self.host = host
        self.port = port
        self.subscripbeTo = subscribe

        self.makeClient(self.clientId,
            on_connect if on_connect else self.on_connect,
            on_message if on_message else self.on_message
            )

    def connect(self):
        self.cli.connect(self.host, self.port)


    def makeClient(self, clientId, on_connect, on_message):
        self.l.i("makeClient")
        self.cli = mqtt.Client( clientId )
        self.cli.on_connect = on_connect
        self.cli.on_message = on_message


    def on_connect(self, client, userdata, flags, rc):
        self.l.i("on_connected with result code [{}]".format(rc) )
        for topic in self.subscripbeTo:
            self.client.subscribe(topic)
        self.ready = True
        return 0

    def on_message(self, client, userdata, msg):
        self.l.i("on_message topic:[{}]    msg:[{}]".formet(
            msg.topic,
            msg.payload
            ))


if __name__=="__main__":
    mqC = myMqttClient(
    "clientStandAlone",
    '127.0.0.1',
    '12883'
    )

    mqC.cli.loop_forever()



import matplotlib.pyplot as plt
import matplotlib.animation as animation
from matplotlib import style
import random
import _thread
import time
from io import BytesIO
from kivy.lang.builder import Builder
from kivy.uix.widget import Widget
from kivy.graphics.texture import Texture
from kivy.uix.image import Image
from kivy.uix.floatlayout import FloatLayout
from kivy.graphics import Color, Rectangle
from kivy.core.image import Image as CoreImage
from kivy.clock import Clock

class myFastPlotWidget(FloatLayout):
    imgTexture = Texture.create(size=(512, 512), colorfmt='RGB',
bufferfmt='ubyte')
    
    def __init__(self, **ka):
        super(myFastPlotWidget,self).__init__(*ka)
        self.pos = [0,0]
        self.size = [512,512]
    
        self.canvas.add(Rectangle(
            texture=self.imgTexture,
            size=(512,512),
            pos=(0,0)
            ))
    
   
class myFastPlot:
    def __init__(self,
        callBacksForData = [], # need to return x,y
        interval = 1000,
        inBuffer = False
        ):
        
        self.cbfd = callBacksForData
        self.interval = interval
        
        if inBuffer:
            self.fig = None
            self.ax1 = None
    
    def initGui(self,gui):
        print("layoutMyFastPlot...")
        Builder.load_file('layoutMyFastPlot.kv')
        
        
    def getWidgetInstance(self):
        self.w = myFastPlotWidget()
        return self.w
    
    def iterOnData(self,i):
        #print("myFastPlot iterOnData",i)
        self.ax1.clear()
        for c in self.cbfd:
            #print("c type ->",type(c))
            self.ax1.plot( *c(), label=str(c.__name__) )
            
        self.ax1.legend()
      
      
    def runBuffMode(self,obj):
        self.objToActOn = obj        
        Clock.schedule_once(self.on_makeBuffIter,self.interval/1000.0)
      
    def on_makeBuffIter(self, a=0):
        self.makeBuffAction(self.objToActOn)
        Clock.schedule_once(self.on_makeBuffIter,self.interval/1000.0)

      
    def makeBuffAction(self,obj):
        obj.clear_widgets()
        buf = self.getBytesBuffer()
        ic = CoreImage(buf, ext="png", filename=("%simage.png"%random.random()))
        img = Image(
            #size_hint = [None,None],
            texture=ic.texture,
            #size = ic.size,
            pos=[0,0],
            keep_ratio = True
            )
        obj.add_widget(img)
    
        
    def getBytesBuffer(self):
        if self.fig == None:
            self.fig = plt.figure()
            self.ax1 = self.fig.add_subplot(1,1,1)
            
        
        self.iterOnData(-1)
        buf = BytesIO()
        plt.savefig(buf, format='png')
        #print("from file 2",buf.__sizeof__(),buf.closed)
        buf.seek(0)
        #plt.savefig('/tmp/figg.png',format='png')
        return buf
        
    def run(self):
        _thread.start_new_thread( self.runIt, () )
        
    
    def runIt(self):    
        self.fig = plt.figure()
        self.ax1 = self.fig.add_subplot(1,1,1)
        self.ani = animation.FuncAnimation(self.fig,self.iterOnData,interval=self.interval)
        plt.show()
    


def anim():
    xs1 = []
    ys1 = []
    for d in range(10):
        xs1.append(d)
        ys1.append(random.randrange(0,10))
        ys1[0] = 0.0
    return xs1,ys1

def anim2():
    xs1 = []
    ys1 = []
    for d in range(10):
        xs1.append(d-20)
        ys1.append(random.randrange(0,10))
        ys1[0] = 0.0
    return xs1,ys1
 
if __name__ == "__main__":
    print("run as test instance")
    g = myFastPlot( [anim,anim2] )
    g.run()
    print("myFastPlot DONE")
    
    while 1:
        time.sleep(1)

 


import xml.etree.cElementTree as ET
import sys
"""
class NOPoint:
	lat = 0.000
	lon = 0.000
	
class NOWPT:
	tagName = "wpt"
	point = None
	time = 0
	name = ""
	md5 = ""
	xml = None
	
class NORTEPT(NOWPT):
	tagName = "rtept"
	

class NORTE:
	tagName = "rte"
	name = ""
	md5 = ""
	xml = None
	rtepts = []
	
class NOTRK:
	tagName = "trk"
	name = ""
	md5 = ""
	xml = None
"""
class XMLHelper:
	textBufSize = 1024
	nsBlank = 'b'
	fileParsed = ""
	nsLocal = None
	
	def gb(self,tag):
		return "%s:%s"%(self.nsBlank, tag)

	def tostring(self, element):
		return ET.tostring(element)

	def parse(self,file):
		try:
			self.fileParsed = file
			self.root = ET.parse(file).getroot()
			self.nsLocal = self.namespaceExtract(open(file).read(self.textBufSize))
			return True
		except:
			print( "XMLHelper file error [%s] not correct!"%file)
			return False
	
	def namespaceExtract(self,text):
		deb = False
		text = text[:self.textBufSize]		
		nsTr = {}		
		if deb:print( "text[%s]"%text)
		for s in text.split("xmlns"):
			if s[0] in [':', '=']:
				if deb:print( "found ns ?")
				if deb:print( "to chk [%s]"%s)
				fRowna = s.find('=')
				fQStart = s.find('"')+1
				fQEnd = fQStart + s[fQStart:].find('"')
				if deb:print( "={} s{} e{}".format(fRowna, fQStart, fQEnd))
				if fRowna < fQStart < fQEnd:
					dd = s.find(':')
					ns = self.nsBlank
					if dd > -1 and dd < fQStart:
						ns = s[dd+1:fQStart-2]
					
					nsUrl = s[fQStart:fQEnd]
					if deb:print( "ns [%s]"%ns)
					if deb:print( "nsUrl [%s]"%nsUrl)
					nsTr[ns] = nsUrl
				
			if deb:print( "-------------------")
		return nsTr
	
	
	def correctDumpXmlFromElement(self,element):
		return self.correctDumpXml( 
			self.tostring(element) 
			)			
	
	def correctDumpXml(self, text):		
		deb = False
		if self.nsLocal == None:
			print( "error no file parsed by object !")
			print( "you need to parse file first to get namespace frome file")
			return None
		dicForReplace = {}
		nsLocal = self.namespaceExtract(text)
		if deb:
			print ("nsFound: %s"%nsLocal)
			print ("in --------------------------")
			print (text)
			print ("-----------------------------")
		for n,u in nsLocal.items():
			key = ' xmlns:{}="{}"'.format(n,u)
			dicForReplace[key] = ''
			
			for ns, url in self.nsLocal.items():
				if url == u:
					if ns == self.nsBlank:
						ns = ''
						n = "%s:"%n
					dicForReplace["<%s"%n] = "<%s"%ns
					dicForReplace["</%s"%n] = "</%s"%ns
					break
				
		if deb:print( "dicForReplace %s"%dicForReplace)
		for f,t in dicForReplace.items():
			if deb:print ("replace from [{}] -> [{}]".format(f,t))
			text = text.replace(f,t)
				
		if deb:
			print( "out ----------------------------")
			print( text)
			print( "out ----------------------------")
		
		return text
		
if __name__ == "__main__":
	print( "----------------------------------\n"*3)
	print( "runing test of xmlhelper ....")
	
	print( "testing namespace extraction from file.....")
	nsE = XMLHelper()
	print( nsE.namespaceExtract( open("/tmp/n.xml").read(1024) ))
	print( "DONE")
	
	print( "----------------------------------\n"*3)
	print( "testing correcting name space from xml.etree.eElementTree.dump.....")
	file = "/tmp/n.xml"
	corSampel = '''
	<ns0:wpt xmlns:ns0="http://www.topografix.com/GPX/1/1" xmlns:ns1="http://www.opencpn.org" lat="15.861516667" lon="-83.891600000">
    <ns0:time>2015-08-11T14:16:16Z</ns0:time>
    <ns0:name>LARRYS ROUTE 1</ns0:name>
    <ns0:sym>triangle</ns0:sym>
    <ns0:type>WPT</ns0:type>
    <ns0:extensions>
      <ns1:guid>22cf0000-8ce9-43c4-a6c0-a1b35a0dffff</ns1:guid>
      <ns1:viz_name>1</ns1:viz_name>
      <ns1:arrival_radius>0.050</ns1:arrival_radius>
      <ns1:waypoint_range_rings colour="#000000" number="-1" step="-1" units="-1" visible="false" />
      <ns1:scale_min_max ScaleMax="0" ScaleMin="2147483646" UseScale="false" />
    </ns0:extensions>
  </ns0:wpt>
'''
	corE = XMLHelper()
	corE.parse(file)
	print( "sample in --------------------")
	print( corSampel)
	print( "sample in END ----------------")
	print( "result -----------------------")
	print( corE.correctDumpXml(corSampel))
	print( "result END -------------------")
	print( "DONE")
	print( "----------------------------------\n"*3)
	
	print( "tests DONE ")


from ot_my_libs.myLoger import myLoger
from ot_my_libs.TimeHelper import *
import pymysql
import os
import time

class mysql_helper:
    """
    Is to do mysql operation fast. Couple of lines and you can go ... 
    ## import library 
      ```python
      from ot_my_libs.mysql_helper import mysql_helper as mhd
      ```
    """
    
        
    is_connected = False
    """**is_connected**: __boolean__ - if connected to mysql"""

    def __init__(self, config = None):
        """
        # Arguments
          **client** _json_: connecting to mysql by `config`
          
        ## config example
          ```json
          config = {
            'host': '192.168.43.1',
            'port': 3306,
            'user': 'ykpu',
            'passwd': 'pimpimpampam',
            'database': 'svoiysh',
            'maxBuf': 1024
          }        
          db = mhd(config)
          ```
        
        **db** __mysql_helper__ now is to your disposition with functions. After init it will `self.connect()`
        """    
    
        if config == None:
            config = {
                'host': 'nex7',
                'port': 3306,
                'user': 'ykpu',
                'passwd': 'pimpimpampam',
                'database': 'svoiysh',
                'maxBuf': 1024
                }

        self.conf = config
        #self.myl = myLoger( "mysql_helper", config.loger)
        self.myl = myLoger( "mysql_helper", {
                'info': True,
                'debug': True,
                'error': True,
                'critical': True
                } )

        self.struct = []
        self.con = None
        self.kur = None

        self.connect()


    def connect(self):
        """To connect if you lost connection"""
        
        self.myl.i("connecting...")
        self.con = pymysql.connect(
            host = self.conf['host'],
            user = self.conf['user'],
            password = self.conf['passwd'],
            db = self.conf['database'],
            cursorclass = pymysql.cursors.DictCursor
            )
        self.myl.i("connected :)")
        self.is_connected = True



    def one_value(self,table_name,what,where="1"):
        """
        To get one value from one record.
        # Arguments
          **table_name** _string_: name of table to work on
          **what** _string_: to put in place of `SELECT XXX FROM ...`
          **where** _string_: to use it it where section 
        *all cap with ` ... LIMIT 1;`*
        
        # Returns 
          __string__ | __int__ | __float__        
        """
        
        tr = self.select(table_name, what, "%s LIMIT 1" % where)
        if len(tr)>0:
            return tr[0][what]
        else:
            return None

    def one_row(self, table_name,where="1"):
        """
        To get one record.
        
        # Arguments
          **table_name** _string_: name of table to work on
          **where** _string_: to use it it where section 
        
        *all cap with ` ... LIMIT 1;`*
        
        # Returns 
          __json__ with your data **example** `{'id': 1487658, 'msg': '34.7', 'entryDate': 1684009661}`        
        """
        
        tr = self.select(table_name, "*", "%s LIMIT 1"%where)
        if len(tr)>0:
            return tr[0]
        else:
            return None

    def select(self, table_name, what="*", where="1"):
        """
        To do `SELECT` on mysql
        
        # Arguments
          **table_name** _string_: name of table to work on
          **what** _string_: to put in place of `SELECT XXX FROM ...`
          **where** _string_: to use it it where section 
        
        *all cap with ` ... LIMIT 1;`*
        
        # Returns 
          __cursor__.fetchall()
        """
    
        q = "select %s from %s where %s" % ( what, table_name, where)
        c = self.con.cursor()
        c.execute(q)
        return c.fetchall()

    def insert(self,table_name, values):
        """
        To **insert** to table
        
        # Arguments
          **table_name** _string_: name of table to work on
          **values** _json_: look format multi ..
        
        # Returns 
          __int__ of inserted `id`
        """
        
        vNames = []
        val = []
        qu = []

        for v in values:
            vNames.append( f"`{v}`" )
            val.append(values[v])
            qu.append( '%s' )

        q0 = "insert into `%s` (%s) values (%s)" % (
                    table_name,
                    ",".join(vNames),
                    ",".join(qu)
                    )
        self.query(q0, val)
        return self.one_value(table_name, "id", "1 order by id desc")

    def insertMulti(self, tableName, data):
        '''
        To **insert** to table
        
        # Arguments
          **table_name** _string_: name of table to work on
          **data** _json_: look format multi ..
            
        # data example
          ```json
          data = {
            'head':['fName0', 'fName1, ....],
            'data':[
              [1 , 2, ...],
              [11, 12, ..],
              ....
              ]
            }
          ```                    
        '''
        
        vals = []
        valq = []
        for v in data['head']:
            vals.append(f"`{v}`")
            valq.append('%s')
        vals = ",".join(vals)
        valq = ",".join(valq)
        q = f"INSERT INTO `{tableName}` ({vals}) VALUES ({valq})"
        self.executemany(q, data['data'])


    def query(self,q0,q1=None):
        c = self.con.cursor()
        if q1 == None:
            res = c.execute(q0)
        else:
            res = c.execute(q0,q1)
        self.con.commit()

    def executemany(self, q0, q1):
        c = self.con.cursor()
        res = c.executemany(q0,q1)
        self.con.commit()
        return res


    def init_table(self):
        if len(self.struct) == 0:
            self.log.critical("you need to overwrite set_struct")
            return False

        for table in self.struct:
            tName = table
            item = self.struct[tName]
            fields = ["id INTEGER PRIMARY KEY"]
            for col in item:
                fields.append( "%s %s" % (col, item[col]) )

            q = "create table %s (%s)" % (tName, ",".join(fields))
            try:
                self.query(q)
            except:
                pass


'''
CREATE TABLE `dbSpeed` (
    `id` int(11) NOT NULL AUTO_INCREMENT,

    `v0` varchar(255) COLLATE utf8_bin NOT NULL,
    `v1` varchar(255) COLLATE utf8_bin NOT NULL,
    `v2` varchar(255) COLLATE utf8_bin NOT NULL,

    `entryDate` int(11) NOT NULL,
    PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_bin
AUTO_INCREMENT=1 ;

'''


if __name__ == "__main__":
    import random

    if 0 :
      mysql = {
        'host': 'nex7',
        'port': 3306,
        'user': 'ykpu',
        'passwd': 'pimpimpampam',
        'database': 'svoiysh',
        'maxBuf': 1024,
        'tableName': 'dbSpeed'
        }
    if 1 :
      mysql = {
        'host': 'hu',
        'port': 3306,
        'user': 'ykpu',
        'passwd': 'pimpimpampam',
        'database': 'svoiysh',
        'maxBuf': 1024,
        'tableName': 'dbSpeed'
        }

    th = TimeHelper()
    db = mysql_helper(mysql)


    print('''what to do ?
1. - test insert speed one by one
2. - test insert speed one by one
    ''')

    print("enter action no:")
    act = input()
    #act = "2"
    print(f"got[{act}]")
    print("clean db first....")
    db.query(f"delete from {mysql['tableName']} where 1;")
    print("    done")
    count = 1000


    if act == '1' or act == '2':
        if act == '1':
            print(f"making test insert speed one by one {count} records")
        elif act == '2':
            print(f"making test insert speed one by one {count} records")

        print("starting test ....")
        bs = th.benStart()
        bsInAvg = 0.0

        if act == '1':
            for r in range(count):
                bsR = th.benStart()

                db.insert(mysql['tableName'], {
                    'v0': random.randrange(0,100),
                    'v1': random.randrange(0,100),
                    'v2': random.randrange(0,100),
                    'entryDate': int(time.time())
                    })

                bsInAvg+= th.benDone(bsR,onlyTime=True)

        elif act == '2':
            dta = {
                'head': ['v0', 'v1', 'v2','entryDate'],
                'data': []
                }

            bsDp = th.benStart()
            d = []
            for r in range(count):
                d.append( [
                    str(random.randrange(0,100)),
                    str(random.randrange(0,100)),
                    str(random.randrange(0,100)),
                    int(time.time())
                    ])
            dta['data'] = d
            bsDp = th.benDone(bsDp, onlyTime=True)

            bQp = th.benStart()
            vals = []
            valq = []
            for v in dta['head']:
                vals.append(f"`{v}`")
                valq.append('%s')
            vals = ",".join(vals)
            valq = ",".join(valq)
            q = f"INSERT INTO `{mysql['tableName']}` ({vals}) VALUES ({valq})"
            bQp = th.benDone(bQp, onlyTime=True)

            bsQe = th.benStart()
            db.executemany(q, dta['data'])
            bsQe = th.benDone(bsQe, onlyTime=True)

            print(f"data preparation {bsDp}")
            print(f"query preparation {bQp}")
            print(f"query {bsQe}")
            bsInAvg = 1.0


        th.benDone(bs, desc="total time", printIt=True)
        avgq = bsInAvg/count
        print(f"avg per query {avgq}")
        print("class test....")
        print("make the same but from mysql_helper")
        b = th.benStart()
        db.insertMulti(mysql['tableName'], dta)
        th.benDone(b, desc="from class", printIt=True)




from ot_my_libs.TimeHelper import *


class myLogColors:
    HEADER =        '\033[95m'
    OKBLUE =        '\033[94m'
    OKCYAN =        '\033[96m'
    OKGREEN =       '\033[92m'
    WARNING =       '\033[93m'
    FAIL =          '\033[91m'
    ENDC =          '\033[0m'
    BOLD =          '\033[1m'
    UNDERLINE =     '\033[4m'
    Black =         '\033[0;30m'
    Dark_Gray =     '\033[1;30m'
    Red =           '\033[0;31m'
    Light_Red =     '\033[1;31m'
    Green =         '\033[0;32m'
    Light_Green	=   '\033[1;32m'
    Brown_Orange =  '\033[0;33m'
    Yellow =        '\033[1;33m'
    Blue =          '\033[0;34m'
    Light_Blue =    '\033[1;34m'
    Purple =        '\033[0;35m'
    Light_Purple =  '\033[1;35m'
    Cyan =          '\033[0;36m'
    Light_Cyan =    '\033[1;36m'
    Light_Gray =    '\033[0;37m'
    White =         '\033[1;37m'

class myLoger:
    def __init__(self, prefix,
            silenc={
                'info': True,
                'debug': True,
                'error': True,
                'critical': True
                },
             defColor="" ):

        self.silenc = silenc
        self.defColor = defColor
        self.th = TimeHelper()
        self.prefix = prefix
        self.i(f'''__init__   {self.defColor}with def color: IT IN{myLogColors.ENDC} ''')

    def m(self, typeMessage, message):
        if self.silenc[typeMessage]:
            try:
                col = self.defColor
                colEnd = ""

                if typeMessage == 'debug':
                    col = myLogColors.OKCYAN
                if col != "":
                    colEnd = myLogColors.ENDC

                print("{0}[{1}] {2} {3} {4} -> ({5})".format(
                    col,
                    self.th.getNiceShortDate(),
                    typeMessage,
                    self.prefix,
                    colEnd,
                    message))
            except:
                print( typeMessage)
                print( message)


        #self.m("erro", message)

    def getSublog(self,parentName):
        return myLoger(
            "{0}/{1}".format(self.prefix, parentName),
            #f"{self.prefix}/{parentName}",
            self.silenc
            )

    def info(self, msg):
        self.i(msg)
    def i(self,message):
        self.m("info", message)


    def debug(self, msg):
        self.d(msg)
        
    def d(self,message):
        self.m("debug",message)

    def error(self, msg):
        self.e(msg)
    def e(self,message):
        self.m("error",message)

    def critical(self,msg):
        self.m("critical", msg)
        
        




from datetime import datetime
import random
import time

class TimeHelper:
	
	def benStart(self):
		try:
			self.benKeys
		except:
			self.benKeys = {}
		key = random.randint(0,999999)*random.randint(0,999999)*random.randint(0,999999)
		self.benKeys[key] = self.getTimestamp(microsec=True)
		return key
		
	def benDone(self,key,desc='',onlyTime=False, printIt = False):
		sec = round((self.getTimestamp(microsec=True)-self.benKeys[key])/1000000.0,6)
		if onlyTime:
			return sec
		if desc == '':
			tr = "%s"%sec
		else:
			tr = "BenDone [%s] Done in [%s] sec"%(desc, sec ) 
		
		if printIt:
			print(tr)
		return tr
	
	def getNiceShortDate(self,timeStamp = None):
		if timeStamp == None:
			timeStamp = self.getTimestamp()
		return datetime.fromtimestamp(float(timeStamp)).strftime("%y%m%d_%H%M%S")
	
	def getDate(self,timeStamp = None, resAsDic = False):
		if timeStamp == None:
			timeStamp = self.getTimestamp()
		
		d = datetime.fromtimestamp(float(timeStamp))
		if resAsDic:
			return {
				'y':int(d.strftime("%y")),
				'm':int(d.strftime("%m")),
				'd':int(d.strftime("%d")),
				'H':int(d.strftime("%H")),
				'M':int(d.strftime("%M")),
				'S':int(d.strftime("%S"))
				}
		else:
			return [ 
				d.strftime("%y"),
				d.strftime("%m"),
				d.strftime("%d"),
				d.strftime("%H"),
				d.strftime("%M"),
				d.strftime("%S")
			]
		
		
	def niceNumbers(self,v):
		if v < 10:
			return "0"+str(v)
		else:
			return str(v)
		
	def getNiceHowMuchTimeItsTaking(self, sec):
		tr = ""
		day = self.niceNumbers( sec // (60*60*24) )
		hh = self.niceNumbers( ( sec // ( 60*60 ) )  % 24 )
		min = self.niceNumbers( (sec // 60) % 60 )
		ss = self.niceNumbers( (sec) % 60 )
		
		return ("%s / %s:%s:%s"%(day,hh,min,ss))
		

	def getTimestampFromStr(self,yyyymmdd_HHMMSS):
		return datetime.strptime(yyyymmdd_HHMMSS, "%Y/%m/%d %H%M%S").strftime('%s')

	

	def getTimestamp(self,microsec=False):
		strf = "%s"
		if microsec:
			strf = "%s%f"
		tr = int(datetime.now().strftime( strf ))
		return tr

	


	def getNiceDateFromTimestamp(self, timeStamp = None, tformat="%Y-%m-%d %T"):
		if timeStamp == None:
			timeStamp = self.getTimestamp()
		return datetime.fromtimestamp(float(timeStamp)).strftime( tformat )

	def getNiceFileNameFromTimestamp(self, timeStamp = None):
		if timeStamp == None:
			timeStamp = self.getTimestamp()
		return datetime.fromtimestamp(float(timeStamp)).strftime("%Y-%m-%d %T").replace("-","_").replace(" ","_").replace(":","_")



from ot_my_libs.myLoger import myLoger as mlog
import sqlite3
import os
from ot_my_libs.FileActions import *

class db_helper:
    
    def __init__(self, pathToDb = None):
        
        try:
            a = self.log.silenc
        except:
            self.log = mlog( "db_helper", 
                silenc={
                    'info': True,
                    'debug': True,
                    'error': True,
                    'critical': True
                    })
    
        
        print("pathToDB is [{}]".format(pathToDb))
        if pathToDb:
            print("pathTo{}".format(pathToDb))
            self.db_at_file = pathToDb
        else:
            self.db_at_file = self.get_filePath()
            print("no path so use{}".format(self.db_at_file))
        
        self.fa = FileActions()
        self.struct = []
        self.conn = None
        
        self.set_struct()
        self.init_table()
        
    # shoud be overrite
    def get_filePath(self):
        self.log.info("get_filePath ~/db_helper.db" )
        return self.fa.t("~/db_helper.db")
    
    # shoud be overrite    
    def set_struct(self):
        print("db_helper is def overrit it !")
        exampleCanBe="""
        self.struct = {
            'wiki' : {
                      'lat':     "float",
                      'lon':     "float",
                      'title':   "text",
                      'summary': "text",
                      'url':     "text",
                      'rank':    "int"                      
                      }
            }

    """
    
    
    
    
    
    
    
    
    
    
    
    
    
    def one_value(self,table_name,what,where="1"):
        tr = self.select(table_name, what, "%s LIMIT 1" % where)
        if len(tr)>0:
            return tr[0][what]
        else:
            return None
    
    def one_row(self, table_name,where="1"):
        tr = self.select(table_name, "*", "%s LIMIT 1"%where)
        if len(tr)>0:
            return tr[0]
        else:
            return None
        
    def select(self, table_name, what="*", where="1"):
        q = "select %s from %s where %s" % ( what, table_name, where)
        c = self.connect()
        res = c.execute(q)
        col_name_list = [tuple[0] for tuple in c.description]
        tr = []
        for row in res:
            a = {}
            for i,col in enumerate(col_name_list):
                a[col] = row[i]
            tr.append(a)
        self.close(c)            
        return tr
    
    def insert(self,table_name, values):
        vNames = []
        val = []
        qu = []
        
        for v in values:
            vNames.append( v )
            val.append(values[v])
            qu.append( "?" )            
    
        q0 = "insert into %s (%s) values (%s)" % (
                    table_name,
                    ",".join(vNames),
                    ",".join(qu)
                    )
        self.query(q0, val)
        return self.one_value(table_name, "id", "1 order by id desc")
    
    def query(self,q0,q1=None):
        c = self.connect()
        if q1 == None:
            res = c.execute(q0)
        else:
            res = c.execute(q0,q1)
        self.conn.commit()        
        self.close(c)
        
    def connect(self):
        self.conn = sqlite3.connect(self.db_at_file)
        return self.conn.cursor()
    
    def close(self,c):
        c.close()
        
    def init_table(self):
        if len(self.struct) == 0:
            self.log.critical("you need to overwrite set_struct")
            return False
        
        for table in self.struct:
            tName = table
            item = self.struct[tName]
            fields = ["id INTEGER PRIMARY KEY"]
            for col in item:
                fields.append( "%s %s" % (col, item[col]) )
             
            q = "create table %s (%s)" % (tName, ",".join(fields))
            try:
                self.query(q)
            except:
                pass
        
        
        
        
    



import os,hashlib
import pickle
import sys
import json

class FileActions:
    """
    File Actions and operations helper set
    """
    
    def getHomeDirectoryForApp(self, appName, platform = 'pc'):
        """
        To make home directory for your app fast on pc / python-for-android. It will look and if not exist `make directory` for your app.
        
        # Arguments
          **appName** _string_: name of your app 
          **values** _json_: look format multi ..
        
        # Returns 
          __string__ `path` to your app directory
          **or**
          None on error
        """
        if platform == 'android':
            dList = [
                "/storage/emulated/0",
                "/storage/self/primary",
                "/sdcard",
                "./"
                ]
            
            try:
                print("it's android. Trying plyer")
                from plyer import storagepath
                print(1)
                sp = str(storagepath.get_external_storage_dir())
                print(2)
                print("got:[",sp,']')
                if len(sp)>=1:
                    dList.append(sp)
            except:
                print("EE - no player or different error :/") 
            
                
        else:
            dList = ['~',"./"]
        
        dirIs = ""
        for dir in dList:
            print("chk home dir",dir)
            if self.isDir(dir):
                print("    dir is pressent")
                homePath = self.join(dir,appName)
                if self.isDir(homePath):
                    print('    and there is a home dir',homePath)
                else:
                    dr = self.mkDir(homePath)
                    print("II - making dir result",dr)
                return homePath
        
        print("EE - no correct home directory !!!")
        
        
        return None
        
    
    def pickleMake(self, obj,fileName, makeAsList = True):
        print("myPickle make file",fileName)
        f = open(fileName,'wb')
        if makeAsList:
            pickle.dump( list(obj), f )
        else:
            pickle.dump( obj, f )
         
            
        try:
            f.close()
        except:
            print("EE - error on close() file :(")
        print("    DONE")
        
    def pickleLoad(self, fileName, asList = True):
        print("myPickle load file",fileName)
        tr = pickle.load( open(fileName,'rb') )
        print("    DONE")
        return tr
    
    def loadFile(self, filePath):
        """
        # Arguments
          **filePath** _string_: path to file to load
                  
        # Returns 
          __array__ content of loaded file line by line
          **or**
          None on error
        """        
        try:
            f = open(self.t(filePath))
        except:
            return None
        
        fOrg = []
        line = ""
        
        while True:
            line = f.readline()
            if line:
               fOrg.append( ("%s"%line).replace("\n", "").replace("\r", "") )
            else:
                break
        #print( "loaded lines: %i from file [%s]" % (len(fOrg),filePath) )
        
        return fOrg
    
    def correctDir(self, path):
        print( "correctDir ", )
        b = os.path.basename(path)
        if b == '':
            print( '	trim end' )
            path = path[:-1]
        print( "	result ",path )
        return self.t(path)
    
    def join(self,adr0, adr1):
        """
        path join solvs `/` **or** `\` problem using os library
        
        # Arguments
          **adr0** _string_: more to root path 
          **adr1** _string_: deeper path
        
        # Returns 
          __string__ `path` joind
          **or**
          None on error
        """
        return self.t(os.path.join(adr0,adr1))
        
    def t(self,path):
        """is to convert `~/` to full path
        # Arguments
          **path** _string_: path to make safe
        
        # Returns 
          __string__ `safe` path
        """
        return os.path.expanduser(path)

    def md5str(self,s):
        return hashlib.md5(s).hexdigest()
    
    def md5sumFast(filename, blocksize=65536):
        hash = hashlib.md5()
        with open(filename, "rb") as f:
            for block in iter(lambda: f.read(blocksize), b""):
                hash.update(block)
        return hash.hexdigest()

    def md5file(self,path):
        path = self.t(path)
        return hashlib.md5("\n".join(self.loadFile(path))).hexdigest()

    def cp(self,srcPath,desPath):
        srcPath = self.t(srcPath)
        desPath = self.t(desPath)
        try:
            self.writeFile(desPath, "\n".join(self.loadFile(srcPath)))
            return 1
        except:
            return 0

    def getDirList(self, path):
        path = self.t(path)
        tr = []
        try:
            d = os.listdir(path)
        except:
            print( "Error FileAction.getDirList [%s] error permisions or no directory"%path )
            return []
        for f in d:
            if self.isDir( os.path.join(path,f) ):
                tr.append(f)
        return tr


    def getFileList(self, path, sortByName=False, filter=None):
        path = self.t(path)
        tr = []
        try:
            d = os.listdir(path)
        except:
            print( "Error FileAction.getDirList [%s] error permisions or no directory"%path )
            return []
        
        for f in d:
            #print("found",f)
            if self.isFile(os.path.join(path,f)):
                #print " file"
                if filter:
                    addIt = False
                    try:
                        abccc = f.index(filter)
                        addIt = True
                    except:
                        pass
                    if addIt:
                        tr.append(f)
                        
                else:
                    tr.append(f)
                
        
        if sortByName:
            tr = sorted(tr)
                
        return tr
        
        
    
    def getListRecursive(self,path, tr = [], files = False, dirs = False):
        path = self.t(path)
        try:
            ol = os.listdir(path)
            for i in ol:
                if self.isDir( self.join( path, i ) ):
                    print( "		in req isDir ",i )
                    if dirs:
                        tr.append(self.join(path,i))
                        print( "				adding" )
                    tr = self.getListRecursive( self.join( path, i ), tr, files, dirs )
                else:
                    if files:
                        tr.append( self.join(path, i) ) 
                
        except:
            print( "error in path ",path )
    
        return tr

    def writeFile(self, path, str):
        f = open(self.t(path),"w")
        f.write(str)
        f.close()

    def mkDir(self, path):
        path = self.t(path)
        try:
            print("mkdir",path)
            os.mkdir(path)
            return True
        except:
            print("FileActions.py - can't creat a directory ",path)
            return False

    def rm(self,filePath):
        os.remove(filePath)

    def mv(self,file,newFile):
        file = self.t(file)
        newFile = self.t(newFile)
        os.rename(file, newFile)

    def ln(self, file, newFile):
        os.symlink(file, newFile)

    def cpB(self, srcFile, desFile):
        srcFile = self.t(srcFile)
        desFile = self.t(desFile)
        fs = open(srcFile,"r")
        fd = open(desFile,"w")
        fd.write( fs.read() )
        fd.close()
        fs.close()


	  
    def getSize(self,path):
        return os.path.getsize(self.t(path))
        
    def getSizeNice(self,path):
        s = self.getSize(path)
        if s>1024000:
            return "%s MB" % round( (s/1024.0/1024.0), 2 )
        elif s>1024:
            return "%s KB" % (s/1024)
        else:
            return "%s bytes" % s
    
    def isFile(self, path):
        return os.path.isfile(self.t(path))

    def isDir(self, path):
        return os.path.isdir(self.t(path))

    
